@@ -138,7 +138,7 @@ class LopperSDT:
         lop_files = []
         sdt_files = []
         for ifile in input_files:
-            if re.search( ".dts$", ifile ):
+            if re.search( ".dts$", ifile ) or re.search( ".dtsi$", ifile ):
                 # an input file is either a lopper operation file, or part of the
                 # system device tree. We can check for compatibility to decide which
                 # it is.
@@ -215,7 +215,7 @@ class LopperSDT:
             # in case there are dtsi files, etc.
             include_paths += " " + str(sdt_file.parent) + " "
             self.dtb = Lopper.dt_compile( fp, input_files, include_paths, force, self.outdir,
-                                          self.save_temps, self.verbose, self.enhanced )
+                                          self.save_temps, self.verbose, self.enhanced, self.permissive )
 
             if self.use_libfdt:
                 self.FDT = Lopper.dt_to_fdt(self.dtb, 'rb')
@@ -377,6 +377,8 @@ class LopperSDT:
     def assist_autorun_setup( self, module_name, module_args = [] ):
         lt = LopperTree()
 
+        if self.verbose > 2:
+            print( "[INFO]: setting up module %s with args: %s" % (module_name,module_args))
         lt['/']['compatible'] = [ 'system-device-tree-v1' ]
         lt['/']['priority'] = [ 3 ]
 
@@ -493,6 +495,13 @@ class LopperSDT:
 
             printer = LopperTreePrinter( True, output_filename, self.verbose )
             printer.strict = not self.permissive
+            try:
+                if self.config['dts']['tabs']:
+                    printer.indent_char = '\t'
+                else:
+                    printer.indent_char = ' '
+            except:
+                pass
             printer.load( tree_to_write.export() )
             printer.exec()
 
@@ -831,7 +840,8 @@ class LopperSDT:
                 exec_tgt = lop_node['exec'].value[0]
                 target_node = lops_tree.pnode( exec_tgt )
                 if self.verbose > 1:
-                    print( "[DBG++]: exec phandle: %s target: %s" % (exec_tgt,target_node))
+                    lop_node.print()
+                    print( "[DBG++]: exec phandle: %s target: %s" % (hex(exec_tgt),target_node))
 
                 if target_node:
                     try:
@@ -1534,15 +1544,24 @@ class LopperSDT:
                 print( "[INFO]: conditional node %s not found, returning" % lop_node.abs_path + "/" + root )
                 return False
 
+            try:
+                cond_select = lop_node["cond_select"]
+            except:
+                cond_select = None
+
             # the subnodes of the conditional lop represent the set of conditions
             # to use. The deepest node is what we'll be comparing
             cond_nodes = conditional_start.subnodes()
             # get the last node
             cond_last_node = cond_nodes[-1]
-            # drop the path to the this conditional lop from the full path of
-            # the last node in the chain. That's the path we'll look for in the
-            # system device tree.
-            cond_path = re.sub( lop_node.abs_path, "", cond_last_node.abs_path)
+
+            if cond_select:
+                cond_path = cond_select.value[0]
+            else:
+                # drop the path to the this conditional lop from the full path of
+                # the last node in the chain. That's the path we'll look for in the
+                # system device tree.
+                cond_path = re.sub( lop_node.abs_path, "", cond_last_node.abs_path)
 
             sdt_tgt_nodes = tree.nodes(cond_path)
             if not sdt_tgt_nodes:
