@@ -180,6 +180,15 @@ class LopperFDT(lopper.base.lopper_base):
         """
         try:
             node = fdt.path_offset( node_prefix )
+            # libfdt returns matches based on partial paths. i.e. if a tree
+            # has /timer@2a810000 and /timer, and you search for /timer, you
+            # will get the node number for /timer@2a810000 .. this causes problems.
+            # so we double check the name, and if it isn't as expected, we do a
+            # slower secondary search to find the proper node.
+            node_name = fdt.get_name( node )
+            if node_name != os.path.basename( node_prefix ):
+                node, matching_nodes = LopperFDT.node_find_by_name( fdt, os.path.basename(node_prefix) + "$" )
+
         except:
             node = -1
 
@@ -426,6 +435,10 @@ class LopperFDT(lopper.base.lopper_base):
         Returns:
             int: The node offset of the created node, if successfull, otherwise -1
         """
+
+        if verbose:
+            print( "[DBG]: lopper fdt: node add: %s" % node_full_path )
+
         paths_to_check = [ node_full_path ]
         n_path = node_full_path
 
@@ -452,9 +465,31 @@ class LopperFDT(lopper.base.lopper_base):
                 # add it
                 for _ in range(MAX_RETRIES):
                     try:
+                        if verbose:
+                            print( "[DBG]:       LopperFDT: node_add: adding node: '%s' parent: %s" % (node_name,node_parent ))
                         node_parent = fdt_dest.add_subnode( node_parent, node_name )
-                    except Exception as e:
-                        fdt_dest.resize( fdt_dest.totalsize() + 1024 )
+                    except FdtException as e:
+                        if e.err == -2:
+                            if verbose:
+                                print( "[DBG]:       LopperFDT: node_add: existing node found, creating temporary placeholder node" )
+
+                            # node exists. This shouldn't happen, unless we've
+                            # hit the prefix bug. Let's delete the node that
+                            # occupied the slot, add ours, and then re-add the node
+
+                            #node_to_remove_add_readd = fdt.path_offset( node_prefix )
+
+                            # we exploit the fact that the holding node starting
+                            # wth loppper<> means that the output export routines
+                            # will ignore it later. And as such, we create the
+                            # node, and the add properties later create it, so
+                            # we get the proper node and don't have to do the
+                            # delete -> add -> delete dance. If this breaks, we
+                            # may have to revisit and do the extra manipulations.
+                            node_name = "lopper%s" % str(_) + node_name
+                        else:
+                            fdt_dest.resize( fdt_dest.totalsize() + 1024 )
+
                         continue
                     else:
                         break
@@ -637,6 +672,8 @@ class LopperFDT(lopper.base.lopper_base):
         props = LopperFDT.node_properties( fdt, nn )
         props_to_delete = []
         for p in props:
+            if verbose:
+                print( "              node sync, considering property: %s %s" % (p.name,p) )
             if node_in['__fdt_phandle__'] and p.name == "phandle":
                 # we just added this, it won't be in the node_in items under
                 # the name name
@@ -644,6 +681,8 @@ class LopperFDT(lopper.base.lopper_base):
             else:
                 props_to_delete.append( p.name )
 
+        if verbose:
+            print( "              node sync: props to delete: %s" % props_to_delete )
         for prop, prop_val in reversed(node_in.items()):
             if re.search( "^__", prop ) or prop.startswith( '/' ):
                 if verbose:
@@ -653,8 +692,13 @@ class LopperFDT(lopper.base.lopper_base):
                 if verbose:
                     print( "          lopper.fdt: node sync: prop: %s val: %s" % (prop,prop_val) )
 
+                try:
+                    qtype = node_in["__{}_type__".format( prop ) ]
+                except:
+                    qtype = None
+
                 # We could supply a type hint via the __{}_type__ attribute
-                LopperFDT.property_set( fdt, nn, prop, prop_val, LopperFmt.COMPOUND )
+                LopperFDT.property_set( fdt, nn, prop, prop_val, LopperFmt.COMPOUND, verbose, qtype )
                 if props_to_delete:
                     try:
                         props_to_delete.remove( prop )
@@ -722,12 +766,17 @@ class LopperFDT(lopper.base.lopper_base):
         # we have a list of: containing dict, value, parent
         dwalk = [ [dct,dct,None]  ]
         node_ordered_list = []
+        node_special_list = []
         while dwalk:
             firstitem = dwalk.pop()
             if type(firstitem[1]) is OrderedDict:
                 node_ordered_list.append( [firstitem[1], firstitem[0]] )
                 for item,value in reversed(firstitem[1].items()):
                     dwalk.append([firstitem[1],value,firstitem[0]])
+            elif type(firstitem[1]) is dict:
+                # type 'dict' are special nodes, that aren't order dependent,
+                # gather them up for future processing.
+                node_special_list.append( firstitem[1] )
             else:
                 pass
 
@@ -751,11 +800,21 @@ class LopperFDT(lopper.base.lopper_base):
                 LopperFDT.node_remove( fdt, nn )
             else:
                 if verbose:
-                    print( "[DBG]:    lopper.fdt: sync: node %s was not found, and could not be remove" % node )
+                    print( "[DBG]:    lopper.fdt: sync: node %s was not found, and could not be removed" % node )
                 # child nodes are removed with their parent, and follow in the
                 # list, so this isn't an error.
                 pass
 
+        for n in node_special_list:
+            if verbose:
+                print( "[DBG]:    lopper.fdt: sync: special node: %s" % n )
+            if n['__path__'] == "/memreserve":
+                memreserve_vals = n['__memreserve__']
+                if verbose:
+                    print( "[DBG]:    lopper.fdt: sync: memreserve: %s" % memreserve_vals )
+                # no swig wrapper, so we do this the hard way
+                libfdt.fdt_add_mem_rsv( fdt._fdt, memreserve_vals[0], memreserve_vals[1] )
+
         # add the nodes
         for n in reversed(node_ordered_list):
             nn = LopperFDT.node_find( fdt, n[0]['__path__'] )
@@ -764,8 +823,14 @@ class LopperFDT(lopper.base.lopper_base):
                 if new_number == -1:
                     print( "[ERROR]:    lopper_fdt: node %s could not be added, exiting" % n[0]['__path__'] )
                     sys.exit(1)
+                else:
+                    if verbose:
+                        print( "[DBG]:    lopper.fdt: sync: node %s added, number is: %s" % (n[0]['__path__'],new_number) )
 
 
+        # were there any __lopper<>__ nodes created ? These were
+        # temporary to work around libfdt prefix issues
+
         # sync the properties
         for n_item in reversed(node_ordered_list):
             node_in = n_item[0]
@@ -774,7 +839,7 @@ class LopperFDT(lopper.base.lopper_base):
             abs_path = node_path
             nn =  node_in['__fdt_number__']
 
-            LopperFDT.node_sync( fdt, node_in, node_in_parent )
+            LopperFDT.node_sync( fdt, node_in, node_in_parent, verbose )
 
     @staticmethod
     def export( fdt, start_node = "/", verbose = False, strict = False ):
@@ -843,6 +908,35 @@ class LopperFDT(lopper.base.lopper_base):
             # cannot start with '/'
             dct[n] = LopperFDT.export( fdt, n, verbose, strict )
 
+        # only when processing the root node, we look to see if there
+        # was a peer /memreserve node. if found, we add it to the exported
+        # dictionary
+        if start_node == "/":
+            memreserve = fdt.num_mem_rsv()
+            mdct = {}
+            if memreserve:
+                mdct["__fdt_number__"] = -1
+                mdct["__fdt_name__"] = "memreserve"
+                mdct["__fdt_phandle__"] = -1
+                mdct["__path__"] = "/memreserve"
+
+                if verbose:
+                    print( "[DBG]:     lopper.fdt export: memreserve: %s" % memreserve )
+                for idx in range(0,memreserve):
+                    mr = fdt.get_mem_rsv(idx)
+                    if verbose:
+                        print( "[DBG]:     lopper.fdt export: memreserve: %s" % (mr))
+
+                    if type(mr) == int:
+                        # we got a single number, which means our start address
+                        # was 0x0, and the value is the range/size
+                        mr = [ 0x0, mr ]
+
+                    mdct["__memreserve__"] = mr
+
+                dct["/memreserve"] = mdct
+
+
         return dct
 
     @staticmethod
@@ -882,7 +976,6 @@ class LopperFDT(lopper.base.lopper_base):
 
         prop_list = LopperFDT.node_properties( fdt, node_path )
         for p in prop_list:
-            # print( "                      export as dict: read: %s" % p.name )
             property_val = LopperFDT.property_get( fdt, node_number, p.name, LopperFmt.COMPOUND )
             prop_dict[p.name] = property_val
             if type_hints:
@@ -1084,6 +1177,15 @@ class LopperFDT(lopper.base.lopper_base):
             node_number = LopperFDT.node_find( fdt, node )
             if node_number == -1:
                 node_number, matching_nodes = LopperFDT.node_find_by_name( fdt, node )
+            else:
+                # libfdt returns matches based on partial paths. i.e. if a tree
+                # has /timer@2a810000 and /timer, and you search for /timer, you
+                # will get the node number for /timer@2a810000 .. this causes problems.
+                # so we double check the name, and if it isn't as expected, we do a
+                # slower secondary search to find the proper node.
+                node_name = fdt.get_name( node_number )
+                if node_name != os.path.basename( node):
+                    node_number, matching_nodes = LopperFDT.node_find_by_name( fdt, os.path.basename(node) + "$" )
 
         if node_number == -1:
             print( "[WARNING]: could not find node %s" % node )
@@ -1317,7 +1419,7 @@ class LopperFDT(lopper.base.lopper_base):
         return val
 
     @staticmethod
-    def property_set( fdt, node_number, prop_name, prop_val, ftype=LopperFmt.SIMPLE, verbose=False ):
+    def property_set( fdt, node_number, prop_name, prop_val, ftype=LopperFmt.SIMPLE, verbose=False, typehint=None ):
         """utility command to set a property in a node
 
         A more robust way to set the value of a property in a node, This routine
@@ -1371,7 +1473,10 @@ class LopperFDT(lopper.base.lopper_base):
             # if sys.getsizeof(prop_val) >= 32:
             for _ in range(MAX_RETRIES):
                 try:
-                    if sys.getsizeof(prop_val) > 32:
+                    if typehint and typehint == LopperFmt.UINT8:
+                        bval = LopperFDT.encode_byte_array([prop_val], 1)
+                        fdt.setprop( node_number, prop_name, bval)
+                    elif sys.getsizeof(prop_val) > 32:
                         fdt.setprop_u64( node_number, prop_name, prop_val )
                     else:
                         fdt.setprop_u32( node_number, prop_name, prop_val )
@@ -1424,10 +1529,13 @@ class LopperFDT(lopper.base.lopper_base):
 
             # list is a compound value, or an empty one!
             if len(prop_val) >= 0:
-                try:
-                    bval = LopperFDT.encode_byte_array_from_strings(prop_val)
-                except Exception as e:
-                    bval = LopperFDT.encode_byte_array(prop_val)
+                if typehint and typehint == LopperFmt.UINT8:
+                    bval = LopperFDT.encode_byte_array(prop_val, 1)
+                else:
+                    try:
+                        bval = LopperFDT.encode_byte_array_from_strings(prop_val)
+                    except Exception as e:
+                        bval = LopperFDT.encode_byte_array(prop_val)
 
                 for _ in range(MAX_RETRIES):
                     try:
@@ -1494,7 +1602,7 @@ class LopperFDT(lopper.base.lopper_base):
 
     @staticmethod
     def dt_compile( dts_file, i_files, includes, force_overwrite=False, outdir="./",
-                    save_temps=False, verbose=0, enhanced = True ):
+                    save_temps=False, verbose=0, enhanced = True, permissive = False ):
         """Compile a dts file to a dtb
 
         This routine takes a dts input file, other dts include files,
@@ -1572,6 +1680,14 @@ class LopperFDT(lopper.base.lopper_base):
                 # nodes.
                 data = re.sub( dts_regex, '', data )
 
+            memres_string = ""
+            memres_regex = re.compile( r'\/memreserve\/(.*)?;' )
+            memres = re.search( memres_regex, data )
+            if memres:
+                if verbose:
+                    print ( "[DBG]: memreserve detected: %s" % memres.group(1) )
+                memres_string = "/memreserve/ %s;" % memres.group(1)
+
             # This captures everything at the start of the file (i.e. a comment block)
             # and puts it into a special pre-mble property in the root node. If we don't
             # do this, and let the comment substituion find it below, we have an invalid
@@ -1597,7 +1713,7 @@ class LopperFDT(lopper.base.lopper_base):
                     data = re.sub( preamble_regex, '/ {' + '\n\n{0}'.format(comment), data, count = 1 )
 
             # put the dts start info back in
-            data = re.sub( '^', '/dts-v1/;\n\n', data )
+            data = re.sub( '^', '/dts-v1/;\n\n%s' % memres_string, data )
 
             # Comment and label substitution
             fp_comments_as_attributes = LopperFDT._comment_translate(data)
@@ -1699,31 +1815,34 @@ class LopperFDT(lopper.base.lopper_base):
                 if label:
                     try:
                         existing_label = labeldict[label]
-                        print( "[ERROR]: duplicate label '%s' detected, processing cannot continue" % label )
-                        if verbose:
-                            print( "[DBG+]: Dumping label dictionary (as processed to error)" )
-                            for l in labeldict:
-                                print( "    %s" % l )
+                        if not permissive:
+                            print( "[ERROR]: duplicate label '%s' detected, processing cannot continue" % label )
+                            if verbose:
+                                print( "[DBG+]: Dumping label dictionary (as processed to error)" )
+                                for l in labeldict:
+                                    print( "    %s" % l )
 
-                            print( "\n[DBG+]: Offending label lines with context:" )
-                            file_as_array = data.splitlines()
-                            pattern = re.compile( r'^\s*?({})\s*?\:(.*?)$'.format(label), re.DOTALL | re.MULTILINE )
-                            match_line = 0
-                            for i,f in enumerate(file_as_array):
-                                m = re.search( pattern, f )
-                                if m:
-                                    try:
-                                        print( "    %s %s" % (i-2,file_as_array[i-2]) )
-                                        print( "    %s %s" % (i-1,file_as_array[i-1]) )
-                                        print( "    %s %s" % (i,file_as_array[i]) )
-                                        print( "    %s %s" % (i+1,file_as_array[i+1]) )
-                                        print( "    %s %s" % (i+2,file_as_array[i+2]) )
-                                    except:
-                                        print( "    %s %s" % (i,file_as_array[i]) )
-
-                                    print( "\n" )
-                        os._exit(1)
+                                print( "\n[DBG+]: Offending label lines with context:" )
+                                file_as_array = data.splitlines()
+                                pattern = re.compile( r'^\s*?({})\s*?\:(.*?)$'.format(label), re.DOTALL | re.MULTILINE )
+                                match_line = 0
+                                for i,f in enumerate(file_as_array):
+                                    m = re.search( pattern, f )
+                                    if m:
+                                        try:
+                                            print( "    %s %s" % (i-2,file_as_array[i-2]) )
+                                            print( "    %s %s" % (i-1,file_as_array[i-1]) )
+                                            print( "    %s %s" % (i,file_as_array[i]) )
+                                            print( "    %s %s" % (i+1,file_as_array[i+1]) )
+                                            print( "    %s %s" % (i+2,file_as_array[i+2]) )
+                                        except:
+                                            print( "    %s %s" % (i,file_as_array[i]) )
 
+                                        print( "\n" )
+                            os._exit(1)
+                        else:
+                            # clobber the dup
+                            labeldict[label] = label
                     except:
                         labeldict[label] = label
 
